% tfrouth - Launches a gui that performs Routh-Hurwitz stability test using the symbolic toolbox.

tfrth = struct;
tfrth.cnst = struct;
tfrth.cnst.bwidth = 300;
tfrth.cnst.bheight = 20;
tfrth.cnst.bvspace = 10;
tfrth.cnst.bhspace = 25;
tfrth.cnst.MARGIN = 2;

% Create Figure
tfrth.fig = figure('Units','Normalized',...
                    'Position',[0.3 0.3 0.4 0.4],...
                    'Resize','off',...
                    'NumberTitle','off',...
                    'Color',[0.9 0.9 0.9],...
                    'Name','Routh Hurwitz',...
                    'Menubar','none',...
                    'DockControls','off'...
                    );
               
set(tfrth.fig,'Units','pixels');
tfrth.figpos = get(tfrth.fig,'Position');
tfrth.figpos = [tfrth.figpos(1:2) tfrth.cnst.bwidth+2*tfrth.cnst.bhspace+2*tfrth.cnst.MARGIN,...
                500];
set(tfrth.fig,'Position',tfrth.figpos);


% Lay out TF entry panels
panelpos = [tfrth.cnst.MARGIN 100-tfrth.cnst.MARGIN,...
                                    tfrth.figpos(3)-2*tfrth.cnst.MARGIN tfrth.figpos(4)-100];
% Button Position vectors (from bottom)
buttonpos(1,:) = [tfrth.cnst.bhspace tfrth.cnst.bvspace tfrth.cnst.bwidth tfrth.cnst.bheight];
for ii = 2:15
   buttonpos(ii,:) = buttonpos(1,:) + [0, (ii-1)*tfrth.cnst.bheight, 0, 0]; 
end
buttonpos(end+1,:) = [buttonpos(1,1), 0, buttonpos(1,3), panelpos(4)-2*tfrth.cnst.bvspace];

% TF Entry Panel 1 - numerator/ denominator
tfrth.TFPanel(1) = uipanel('Parent',tfrth.fig,...
                        'Units','pixels',...
                        'Position',panelpos,...
                        'Title','Enter Characteristic Equation:',...
                        'BorderType','beveledin',...
                        'Visible','on',...
                        'BackgroundColor',get(tfrth.fig,'Color'));
    instructions = {'Enter the characteristic equation using a vector of polynomial coefficients, or in symbolic form with complex variable ''s''. Enter ALL symbolic parameters in the box labeled ''Enter Symbolic Parameters.''',...
                    '                                                      ',...
                    'ex: For (s^2 + 3*kp*s + ki^2):',...
                    '     enter ''[1, 3*kp, ki^2]''  in the Characteristic Equation box ',...
                    '     and ''kp ki'' in the symbolic variables text box.',...
                    'OR: Type in (s^2 + 3*kp*s + ki^2) in the Characteristic Equation box and ''kp ki''  in the symbolic variables text box.',...
                    '                                                      ',...
                    'ex: The following are all equivalent:',...
                    '         ''(s^2 + 7*s + 12),    [1 7 12],    and   (s+4)*(s+3)''.'};
                
                    tfrth.ndinstructions = uicontrol('Parent',tfrth.TFPanel(1),...
                                            'Units','pixels',...
                                            'Position',buttonpos(end,:) + [0, buttonpos(10,2), 0, -buttonpos(10,2)],...
                                            'Style','text',...
                                            'String','',...
                                            'BackgroundColor',get(tfrth.fig,'Color'),...
                                            'HorizontalAlignment','left');
                    set(tfrth.ndinstructions,'String',textwrap(tfrth.ndinstructions,instructions));
                    tfrth.symtext = uicontrol('Parent',tfrth.TFPanel(1),...
                                            'Units','pixels',...
                                            'Position',buttonpos(7,:)+ [0, 0, -tfrth.cnst.bwidth/2, tfrth.cnst.bheight/2],...
                                            'Style','text',...
                                            'String','Enter Symbolic Parameters (For example:)',...
                                            'BackgroundColor',get(tfrth.fig,'Color'));
                    tfrth.symbox = uicontrol('Parent',tfrth.TFPanel(1),...
                                            'Units','pixels',...
                                            'Position',buttonpos(7,:)+ [tfrth.cnst.bwidth/2, 0, -tfrth.cnst.bwidth/2, tfrth.cnst.bheight/2],...
                                            'Style','edit',...
                                            'String','k',...
                                            'BackgroundColor',[1 1 1]);
                    tfrth.dentext = uicontrol('Parent',tfrth.TFPanel(1),...
                                            'Units','pixels',...
                                            'Position',buttonpos(6,:)+[0 0 0,-5],...
                                            'Style','text',...
                                            'String','Characteristic Equation',...
                                            'BackgroundColor',get(tfrth.fig,'Color'));
                    tfrth.denbox = uicontrol('Parent',tfrth.TFPanel(1),...
                                            'Units','pixels',...
                                            'Position',buttonpos(4,:)+[0, 0, 0, tfrth.cnst.bheight],...
                                            'Style','edit',...
                                            'String','[1]',...
                                            'BackgroundColor',[1 1 1]);
                    tfrth.routhbutton = uicontrol('Parent',tfrth.TFPanel(1),...
                                             'Style','Pushbutton',...
                                             'String','Routh-Hurwitz',...
                                             'Units','pixels',...
                                             'Position',buttonpos(2,:)+[0, 0, 0, tfrth.cnst.bheight],...
                                             'Value',1,...
                                             'Callback',...
                                            ['commandwindow; RH = routheval(parsePoly(get(tfrth.denbox,''String''),get(tfrth.symbox,''String'')));'...
                                             'disp(''Routh-Hurwitz Matrix:''), pretty(RH), figure(tfrth.fig);']);
 % Control Panel and Buttons
 
 tfrth.ControlPanel = uipanel('Parent',tfrth.fig,...
                              'Units','pixels',...
                              'Position',[panelpos(1), tfrth.cnst.MARGIN,...
                                          panelpos(3), tfrth.figpos(4)-panelpos(4)-2*tfrth.cnst.MARGIN],...
                              'BorderType','line',...
                              'BackgroundColor',get(tfrth.fig,'Color'),...
                              'Visible','on');
 
                        tfrth.closebutton = uicontrol('Parent',tfrth.ControlPanel,...
                                                    'Style','Pushbutton',...
                                                    'String','Close',...
                                                    'Units','pixels',...
                                                    'Position',buttonpos(2,:)+[0, 0, 0, tfrth.cnst.bheight],...
                                                    'Value',1,...
                                                    'Callback','close(gcf)');

clear instructions buttonpos panelpos