% function exp_fig
%   -lays out the GUI Panels and controls for VirtualLab and SIMLab Experiments
%
%  Farid Golnaraghi 1993, 1999, 2002, 2007
% The MechWorks Software Inc., Waterloo, Ont., Canada
% Version 2002 - First Trial
%=============================================================
% Author: Johannes Minor
% SIMLab, Experiment 1: Speed control GUI
% Last Modified: July 4, 2007
%=============================================================

globalvars				% Initialization Script File

%===================================================================
%Initialize some variables
%===================================================================

output.signals.values=zeros(100);
output.time=zeros(100);
input.signals.values=zeros(100);
input.time=zeros(100);
vars.pTimer = timer;
delete(vars.pTimer);

cnst.BW_BG = 0.18;   % Button size constants
cnst.BW_SM = 0.12;
cnst.MARGIN = 0.02;
vars.printfig = -1;

cnst.axpos1 = [5*cnst.MARGIN 5*cnst.MARGIN 1-10*cnst.MARGIN 1-10*cnst.MARGIN];
cnst.axpos2 = [5*cnst.MARGIN 0.5+5*cnst.MARGIN 1-10*cnst.MARGIN 0.5-10*cnst.MARGIN];
guiel.hAX(1) = -1;
guiel.hAX(2) = -1;
guiel.hAX(3) = -1;
guiel.tfPanel = -1;

vars.L = 0;
vars.R = 0;
vars.J = 0;
vars.b = 0;
vars.Kb = 0;
vars.Km = 0;
vars.P = 0;
vars.I = 0;
vars.D = 0;

vars.dropheight = 24.5;
vars.armradius = 508;



% Experiment Select Vars
%   - response_type
%       speed | position | frequency
%   - simfilename
%       speedctrl_sim | positionctrl_sim | openloopstep_sim | frequency_sim
%       speedctrl_virt | positionctrl_virt | openloopstep_virt |
%       frequency_virt
%       | controldesign_sim

%===================================================================
%=====MODEL CONTROL PANEL===========================================
%===================================================================
guiel.cPanel(1) = uipanel('Parent', guiel.ControlPanel,'BorderType','etchedin',...
    'BackgroundColor',cnst.LIGHTGREY,'Units','pixels','Position',...
    [cnst.hmargin cnst.cpheight-cnst.panel2BHeight-cnst.vmargin cnst.panelWidth cnst.panel2BHeight],...
    'Title','Model Control');

        % Tool Tips
        guiel.modelTt = 'Open the Simulink Model window to edit the block parameters.';
        guiel.plotTt = ['Plot the ' vars.response_type ' response of the DC Motor.'];
        guiel.plot2figTt = ['Export the ' vars.response_type ' response plot to a MATLAB figure.'];
        guiel.plot2samefigTt = ['Draw multiple response curves on the same axes for comparison when using Plot to Figure.'];
        guiel.cursorTt = 'Data Cursor: Click Mouse Cursor on Graph or use Arrow Keys';
        guiel.zoomoutTt = 'Zoom Out';
        guiel.zoominTt = 'Zoom In: Select Zoom Area';

        % Text Box
        uicontrol('Parent',guiel.cPanel(1),'Style','text','Units','pixels',...
            'Position',[cnst.bhspace 2*cnst.bvspace+cnst.bheight cnst.bwidth*0.66 cnst.bheight],...
            'String','Simulation Time (s):','fontsize',10,'FontWeight','Demi',...
            'BackgroundColor',cnst.LIGHTGREY);
        % Simulation Time Edit Box
        guiel.hSimtime = uicontrol('Parent',guiel.cPanel(1),'Style','edit','Units','pixels',...
            'Position',[cnst.bhspace+cnst.bwidth*0.66 2*cnst.bvspace+cnst.bheight cnst.bwidth*0.34 cnst.bheight],...
            'String',num2str(vars.simtime),'fontsize',10,'FontWeight','Demi',...
            'Callback','vars.simtime = str2num(get(guiel.hSimtime,''String''));',...
            'BackGroundColor',WHITE);
        % Button to launch Simulink Window
        guiel.hPB(1) = uicontrol('Parent', guiel.cPanel(1),'Style','PushButton','TooltipString',guiel.modelTt,...
            'Units','pixels','Position',[cnst.bhspace cnst.bvspace cnst.bwidth cnst.bheight],...
            'String','Enter Model Parameters','CallBack','eval(codeblk.OPEN_MDL_FCN);',...
            'fontsize',10,'FontWeight','Demi');
        
%===================================================================
%=====TRANSFER FUNCTION DISPLAY PREPARATION=========================
%===================================================================        
if strcmp(vars.simfilename(end-2:end),'sim')
    cnst.axpos1 = [5*cnst.MARGIN 5*cnst.MARGIN 1-10*cnst.MARGIN 1-10*cnst.MARGIN];
    cnst.axpos2 = [5*cnst.MARGIN 0.5+2*cnst.MARGIN 1-10*cnst.MARGIN 0.5-5*cnst.MARGIN];
    cnst.axpos3 = [0 0 1 1];
    cnst.tfPanelpos = [5*cnst.MARGIN 0.99 1-10*cnst.MARGIN 0.01];
    
    guiel.tfPanel = uipanel('Parent',guiel.DisplayPanel,'BackgroundColor',cnst.OFFWHITE,'Units','Normalized',...
        'Position',cnst.tfPanelpos,'Visible','off','BorderType','none','Clipping','on',...
        'DeleteFcn','');
    
    guiel.hAX(3) = axes('Parent',guiel.tfPanel,'Color',cnst.OFFWHITE,'Layer',...
            'top','Xlim',[0 1],'YLim',[0 1],'GridLineStyle','none','Units','Normalized',...
            'XTick',[],'YTick',[],'Box','off','Visible','off','Position',cnst.axpos3);
    
    List = {'{SIMLab Tools}','1. Simplified 2nd Order CL TF','2. 3rd Order CL TF',...
            '3. Simplified 2nd Order OL TF','4. 3rd Order OL TF','Show Simulation Parameters','Gear Head Calculator'};
    
    if ishandle(guiel.TFSelectBox)
        delete(guiel.TFSelectBox);
    end
    
    guiel.TFSelectBox = uicontrol('Parent',guiel.DisplayPanel,'Style','popupmenu','String',List,'Units','pixels',...
        'Position',[2 cnst.displayheight-cnst.bheight-4 cnst.bwidth cnst.bheight],'Visible','off',...
        'Callback','eval(codeblk.showTF);');
   
    clear List;
    
    codeblk.showTF = ['switch get(guiel.TFSelectBox,''Value'') '...
                           'case 1, kiddies = get(guiel.hAX(3),''Children''); delete(kiddies); clear kiddies; '...
                                    'set(guiel.tfPanel,''Visible'',''off'',''Position'',cnst.tfPanelpos); set(guiel.hAX(1),''Position'',cnst.axpos1);'...
                                     'if ishandle(guiel.hAX(2)) set(guiel.hAX(2),''Position'',cnst.axpos2); end;'...
                            'case 2, showTF(vars.symbolTF.tf2ndOsym,vars.numericTF.tf2ndOstr,guiel.tfPanel,guiel.hAX(2),guiel.hAX(1),cnst.displayheight,cnst.displaywidth);'...
                            'case 3, showTF(vars.symbolTF.tf3rdOsym,vars.numericTF.tf3rdOstr,guiel.tfPanel,guiel.hAX(2),guiel.hAX(1),cnst.displayheight,cnst.displaywidth);'...
                            'case 4, showTF(vars.symbolTF.motor2OLsym,vars.numericTF.motor2OL,guiel.tfPanel,guiel.hAX(2),guiel.hAX(1),cnst.displayheight,cnst.displaywidth);'...
                            'case 5, showTF(vars.symbolTF.motor3OLsym,vars.numericTF.motor3OL,guiel.tfPanel,guiel.hAX(2),guiel.hAX(1),cnst.displayheight,cnst.displaywidth);'...
                            'case 7, showTF(guiel.tfPanel,guiel.hAX(2),guiel.hAX(1),cnst.displayheight,cnst.displaywidth);'...
                            'case 6, showTF([vars.L vars.R vars.J vars.b vars.Kb vars.Km vars.P vars.I vars.D],'...
                                    'guiel.tfPanel,guiel.hAX(2),guiel.hAX(1),cnst.displayheight,cnst.displaywidth);'...
                       'end;'];
    set(guiel.hPB(1),'Callback',[get(guiel.hPB(1),'Callback') ' set(guiel.TFSelectBox,''Visible'',''on'');']);
end
%===================================================================
%=====PLOT CONTROLS=================================================
%===================================================================
guiel.cPanel(2) = uipanel('Parent', guiel.ControlPanel,'BorderType','etchedin',...
    'BackgroundColor',cnst.LIGHTGREY,'Units','pixels','Position',...
    [cnst.hmargin cnst.cpheight-cnst.panel3BHeight-cnst.panel2BHeight-2*cnst.vmargin cnst.panelWidth cnst.panel3BHeight],...
    'Title','Plotting');
        % Simulation Button
        guiel.hPB(2)=uicontrol(guiel.cPanel(2),'Style','PushButton','Units','pixels',...
            'TooltipString',guiel.plotTt,...
            'Position',[cnst.bhspace 3*cnst.bvspace+2*cnst.bheight cnst.bwidth cnst.bheight],...
            'String','Run Simulation','fontsize',10,'FontWeight','Demi',...
            'CallBack','eval(codeblk.CLOSE_MDL_FCN); set(guiel.cPanel(4),''Visible'',''on'');set(guiel.APPWINDOW,''Resize'',''off''); primePlot;',...
            'Enable','off');
        % Print To Figure
        guiel.hPB(3)=uicontrol(guiel.cPanel(2),'Style','PushButton','Units','pixels',...
            'TooltipString',guiel.plot2figTt,...
            'Position',[cnst.bhspace 2*cnst.bvspace+cnst.bheight cnst.bwidth cnst.bheight],...
            'String','Plot to Figure','fontsize',10,'FontWeight','Demi',...
            'FontWeight','Demi','CallBack','eval(codeblk.CLOSE_MDL_FCN); print2figg',...
            'Enable','off');
        guiel.plot2samefig = uicontrol(guiel.cPanel(2),'Style','checkbox','Units','pixels',...
            'TooltipString',guiel.plot2samefigTt,'BackgroundColor',cnst.LIGHTGREY,...
            'Position',[cnst.bhspace cnst.bvspace cnst.bwidth cnst.bheight],...
            'String','Reuse Axes','fontsize',10,'FontWeight','Demi',...
            'FontWeight','Demi');

%===================================================================
%=====ANIMATION CONTROLS============================================
%===================================================================
% Default: Set up for Speed Control Plots
guiel.cPanel(3) = uipanel('Parent',guiel.ControlPanel,'Units','pixels','BorderType','none',...
        'BackgroundColor',cnst.LIGHTGREY,'Title','','Visible','off','Position',...
        [cnst.hmargin cnst.cpheight-2*cnst.panel2BHeight-cnst.panel4BHeight-3*cnst.vmargin cnst.panelWidth cnst.panel4BHeight]);
    
vars.plot_units = 'Speed (rad/s)';
guiel.hAX(1) = axes('Parent',guiel.DisplayPanel,'Units','Normalized','ButtonDownFcn','',...
                'Position',cnst.axpos1,'Visible','Off');
guiel.hAX(2) = -1;


if strcmp(vars.response_type,'Position') == 1
        % Animation Controls
        set(guiel.cPanel(2),'Visible','off');
        delete(guiel.cPanel(3));
        guiel.cPanel(3) = uipanel('Parent',guiel.ControlPanel,'Units','pixels','BorderType','etchedin',...
            'BackgroundColor',cnst.LIGHTGREY,'Title','Animation','Position',...
            [cnst.hmargin cnst.cpheight-cnst.panel2BHeight-cnst.panel4BHeight-3*cnst.vmargin cnst.panelWidth cnst.panel4BHeight]);

        guiel.hPB(4)=uicontrol(guiel.cPanel(3),'Style','PushButton','Units','pixels',...
            'Position',[cnst.bhspace 4*cnst.bvspace+3*cnst.bheight cnst.bwidth cnst.bheight],...
            'String','Run Simulation','fontsize',10,'FontWeight','Demi',...
            'CallBack','eval(codeblk.CLOSE_MDL_FCN); set(guiel.APPWINDOW,''Resize'',''off''); set(guiel.cPanel(4),''Visible'',''on''); eval(codeblk.animate_inGUI); ','Enable','off');
        guiel.hPB(5)=uicontrol(guiel.cPanel(3),'Style','PushButton','Units','pixels',...
            'Position',[cnst.bhspace 3*cnst.bvspace+2*cnst.bheight cnst.bwidth cnst.bheight],...
            'String','Stop Simulation','fontsize',10,'FontWeight','Demi','CallBack',...
            'hold off;set(guiel.APPWINDOW,''Resize'',''on''); eval(codeblk.stopTimer); clear anim hght rd;','Enable','off');
         guiel.hPB(6)=uicontrol(guiel.cPanel(3),'Style','PushButton','Units','pixels',...
            'Position',[cnst.bhspace 2*cnst.bvspace+cnst.bheight cnst.bwidth cnst.bheight],...
            'String','Plot to Figure','fontsize',10,'FontWeight','Demi',...
            'CallBack','print2figg','Enable','off');
         set(guiel.plot2samefig,'Parent',guiel.cPanel(3));
        
         guiel.hAX(2)=axes('Parent',guiel.DisplayPanel,'Units','Normalized',...
            'Position',cnst.axpos2,'Visible','Off');

        primeAnimateExp2

        if strcmp(vars.simfilename,'positionctrl_virt') || strcmp(vars.simfilename,'positionctrl_sim')
            codeblk.animate_inGUI = ['eval(codeblk.stopTimer);'...
                                'vars.enableanimation=1;clear hPL; gcf; axes(guiel.hAX(2)); animate_exp2;'];
            delete(guiel.hPB(5));
            guiel.hPB(5)=uicontrol(guiel.cPanel(3),'Style','PushButton','Units','pixels',...
            'Position',[cnst.bhspace 3*cnst.bvspace+2*cnst.bheight cnst.bwidth cnst.bheight],...
            'String','Stop Simulation','Enable','off','fontsize',10,'FontWeight','Demi','CallBack',...
            'hold off; set(guiel.APPWINDOW,''Resize'',''on''); vars.enableanimation = 0; eval(codeblk.stopTimer); clear anim hght rd;');
        else
            codeblk.animate_inGUI = ['eval(codeblk.stopTimer);'...
                                'vars.enableanimation=1; gcf; axes(guiel.hAX(2)); primeModel_5'];
           guiel.axpos2 = [5*MARGIN 0.5+5*MARGIN 1-10*MARGIN 0.5-10*MARGIN];
           [throw1 throw2 throw3 guiel.hAX(2)] = drawModel(guiel.hAX(2),[vars.armradius vars.dropheight],guiel.axpos2);
           clear throw1 throw2 throw3;
           if strcmp(vars.simfilename,'controldesign_sim')
                List = {'{SIMLab Tools}','1. Simplified 2nd Order CL TF','2. 3rd Order CL TF',...
                '3. Simplified 2nd Order OL TF','4. 3rd Order OL TF','Show Simulation Parameters','Gear Head Calculator','Modify Puck Drop Setup'};
                codeblk.showTF = ['switch get(guiel.TFSelectBox,''Value'') '...
                               'case 1, kiddies = get(guiel.hAX(3),''Children''); delete(kiddies); clear kiddies; '...
                                        'set(guiel.tfPanel,''Visible'',''off'',''Position'',cnst.tfPanelpos); set(guiel.hAX(1),''Position'',cnst.axpos1);'...
                                         'if ishandle(guiel.hAX(2)) set(guiel.hAX(2),''Position'',cnst.axpos2); end;'...
                                'case 2, showTF(vars.symbolTF.tf2ndOsym,vars.numericTF.tf2ndOstr,guiel.tfPanel,guiel.hAX(2),guiel.hAX(1),cnst.displayheight,cnst.displaywidth);'...
                                'case 3, showTF(vars.symbolTF.tf3rdOsym,vars.numericTF.tf3rdOstr,guiel.tfPanel,guiel.hAX(2),guiel.hAX(1),cnst.displayheight,cnst.displaywidth);'...
                                'case 4, showTF(vars.symbolTF.motor2OLsym,vars.numericTF.motor2OL,guiel.tfPanel,guiel.hAX(2),guiel.hAX(1),cnst.displayheight,cnst.displaywidth);'...
                                'case 5, showTF(vars.symbolTF.motor3OLsym,vars.numericTF.motor3OL,guiel.tfPanel,guiel.hAX(2),guiel.hAX(1),cnst.displayheight,cnst.displaywidth);'...
                                'case 7, ans = showTF(vars.armradius,guiel.tfPanel,guiel.hAX(2),guiel.hAX(1),cnst.displayheight,cnst.displaywidth,''exp5'');'...
                                'case 6, showTF([vars.L vars.R vars.J vars.b vars.Kb vars.Km vars.P vars.I vars.D],'...
                                        'guiel.tfPanel,guiel.hAX(2),guiel.hAX(1),cnst.displayheight,cnst.displaywidth);'...
                                'case 8, ans = showTF([vars.armradius vars.dropheight],guiel.tfPanel,guiel.hAX(2),guiel.hAX(1),cnst.displayheight,cnst.displaywidth);'...
                           'end;'];
                 set(guiel.TFSelectBox,'String',List);
                 clear List
           end
            
        end

        set(guiel.hPB(1),'Callback',...
        [get(guiel.hPB(1),'Callback') ' set(guiel.hPB(4),''Enable'',''on'');'...
                                        'set(guiel.hPB(5),''Enable'',''on'');'...
                                        'set(guiel.hPB(6),''Enable'',''on'');'...
                                        'set(guiel.hPB(2),''Enable'',''on'');'...
                                        'set(guiel.hPB(3),''Enable'',''on'');']);

        delete(guiel.hAX(1));

        cnst.axpos1 = [5*cnst.MARGIN 5*cnst.MARGIN 1-10*cnst.MARGIN 0.5-5*cnst.MARGIN];
        
        guiel.hAX(1)=axes('Parent',guiel.DisplayPanel,'Units','Normalized','ButtonDownFcn','',...
                        'Position',cnst.axpos1,'Visible','Off');

        vars.plot_units = 'Position (deg)';
 
%===================================================================
%=====GAIN/PHASE CONTROLS===========================================
%===================================================================
else if strcmp(vars.response_type,'Frequency') == 1

        delete(guiel.cPanel(3));
        
        guiel.cPanel(3) = uipanel('Parent',guiel.ControlPanel,'Units','pixels','BorderType','etchedin',...
            'BackgroundColor',cnst.LIGHTGREY,'Title','Gain and Phase Calculator',...
            'Position',[cnst.hmargin cnst.cpheight-cnst.panel2BHeight-cnst.panel3BHeight-cnst.panel4BHeight-3*cnst.vmargin cnst.panelWidth cnst.panel4BHeight]);
        
        uicontrol(guiel.cPanel(3),'Style','text','Units','pixels',...
            'Position',[cnst.bhspace 4*cnst.bvspace+3*cnst.bheight 3*cnst.bwidth/4 cnst.bheight],...
            'Foreground',0.55*BLACK,'Background',cnst.LIGHTGREY,'HorizontalAlignment','Left',...
            'String','Enter frequency(rad/s):','FontWeight','Demi');

        guiel.GP(1)=  uicontrol(guiel.cPanel(3),'Style','edit','Units','pixels',...
                'Position',[cnst.bhspace+3*cnst.bwidth/4 4*cnst.bvspace+3*cnst.bheight cnst.bwidth/4 cnst.bheight],...
                'Background',WHITE,'HorizontalAlignment','Center',...
                'fontsize',10,'FontWeight','Demi');

        guiel.GP(2)=  uicontrol(guiel.cPanel(3),'Style','PushButton','Units','pixels',...
                'Position',[cnst.bhspace 3*cnst.bvspace+2*cnst.bheight cnst.bwidth cnst.bheight],...
                'String','Calculate','fontsize',10,'FontWeight','Demi','CallBack','eval(codeblk.GPwin);','Enable','off');
    
        uicontrol(guiel.cPanel(3),'Style','text','Units','pixels',...
            'Position',[cnst.bhspace 2*cnst.bvspace+cnst.bheight cnst.bwidth/2-cnst.bhspace/2 cnst.bheight],...
            'Foreground',0.55*BLACK,'Background',cnst.LIGHTGREY,...
            'String','Gain','FontWeight','Demi');

        uicontrol(guiel.cPanel(3),'Style','text','Units','pixels',...
            'Position',[2*cnst.bhspace+cnst.bwidth/2 2*cnst.bvspace+cnst.bheight cnst.bwidth/2-cnst.bhspace/2 cnst.bheight],...
            'Foreground',0.55*BLACK,'Background',cnst.LIGHTGREY,...
            'String','Phase(deg)','FontWeight','Demi');
        
        codeblk.GPwin=['clear vars.sizetime vars.cnt vars.period vars.lastcycle vars.output_lastcycle vars.input_lastcycle;'...
            'vars.period=2*pi/str2num(get(guiel.GP(1),''String''));vars.sizetime = length(output.time);vars.cnt=1;'...
            'for (ii=1:vars.sizetime) if (output.time(vars.sizetime)-output.time(ii))<vars.period vars.lastcycle(vars.cnt)=ii;vars.cnt=vars.cnt+1;'...
            'end; end; clear ii;'... 
            'vars.output_lastcycle=output.signals.values(vars.lastcycle); vars.input_lastcycle=input.signals.values(vars.lastcycle);'...
            '[vars.output_peak,vars.op_pk_index]=max(vars.output_lastcycle); [vars.input_peak,vars.ip_pk_index]=max(vars.input_lastcycle);'...
            'vars.gain=vars.output_peak/vars.input_peak;'...
            'vars.phase=360*(input.time(vars.lastcycle(vars.ip_pk_index))-output.time(vars.lastcycle(vars.op_pk_index)))/vars.period;'...
            'uicontrol(guiel.cPanel(3),''Style'',''Text'',''Units'',''pixels'',''string'','...
            'num2str(vars.gain),''Background'',CYAN,'...
            '''Position'', [cnst.bhspace cnst.bvspace cnst.bwidth/2-cnst.bhspace/2 cnst.bheight]),'...
            'uicontrol(guiel.cPanel(3),''Style'',''Text'',''Units'',''pixels'',''string'','...
            'num2str(vars.phase),''Background'',CYAN,'...
            '''Position'', [2*cnst.bhspace+cnst.bwidth/2 cnst.bvspace cnst.bwidth/2-cnst.bhspace/2 cnst.bheight]),'...
            ];
        
        set(guiel.hPB(1),'Callback',...
        [get(guiel.hPB(1),'Callback') ' set(guiel.GP(1),''Enable'',''on'');'...
                                        'set(guiel.GP(2),''Enable'',''on'');'...
                                        'set(guiel.hPB(2),''Enable'',''on'');'...
                                        'set(guiel.hPB(3),''Enable'',''on'');']);
    else
        set(guiel.hPB(1),'Callback',...
        [get(guiel.hPB(1),'Callback') ' set(guiel.hPB(2),''Enable'',''on'');'...
                                        'set(guiel.hPB(3),''Enable'',''on'');']);
    end
end

%===================================================================
%=====CURSOR AND ZOOM CONTROLS======================================
%===================================================================
guiel.cPanel(4) = uipanel('Parent',guiel.DisplayPanel,'Units','pixels','BorderType','none',...
            'BackgroundColor',OFFWHITE,'Title','','Visible','off',...
            'Position',[3*cnst.displaywidth/4-90/2 cnst.bvspace 90 30]); drawnow

guiel.CursorWindow = -1;
guiel.cursorTxt = -1;
guiel.dragBox = -1;
guiel.CursorBar = -1;

iczoomin = imread('zoomin.jpg');
iczoomout = imread('zoomout.jpg');
iccursor = imread('cursor.jpg');

% Zoom In
guiel.zoomzoom=uicontrol(guiel.cPanel(4),'Style','PushButton','Units','pixels',...
	'Position',[0 0 cnst.icwidth cnst.icwidth],'Enable','on',...
    'CData',iczoomin,'TooltipString',guiel.zoominTt,'CallBack',...
    ['vars.axlims = [get(guiel.hAX(1),''XLim'') get(guiel.hAX(1),''YLim'')];'...
        'set(guiel.APPWINDOW,''PointerShapeCData'',cnst.MagnifyingGlass);'...
        'set(guiel.APPWINDOW,''WindowButtonMotionFcn'',''eval(codeblk.ZoomPointerControl);'');'...
        'set(guiel.hAX(1),''ButtonDownFcn'',''boxReady'');set(guiel.zoomzoom,''Enable'',''off'');']);
  % Zoom Out
guiel.zazoomzoomzoom=uicontrol(guiel.cPanel(4),'Style','PushButton','Units','pixels',...
	'Position',[cnst.icwidth 0 cnst.icwidth cnst.icwidth],'Enable','off',...
    'CData',iczoomout,'TooltipString',guiel.zoomoutTt,'CallBack',...
    ['set(guiel.zoomzoom,''Enable'',''on''); set(guiel.zazoomzoomzoom,''Enable'',''off'');'...
    'vars.axlims = [0 vars.simtime 1.05*[ min(anim.pos(find(anim.pos == min(anim.pos),1,''first'')),anim.ip(find(anim.ip == min(anim.ip),1,''first''))),'...
            '1.05*max(anim.pos(find(anim.pos == max(anim.pos),1,''first'')),anim.ip(find(anim.ip == max(anim.ip),1,''first'')))] ];'...
            'set(guiel.hAX(1),''XLim'',vars.axlims(1:2),'...
            '''YLim'',vars.axlims(3:4),'...
            '''GridLineStyle'','':'',''ButtonDownFcn'','''');'...
      'set(guiel.APPWINDOW,''WindowButtonMotionFcn'','''',''WindowButtonUpFcn'','''',''KeyPressFcn'','''',''Pointer'',''arrow'');']);

  % Data Cursor
guiel.cursorPB=uicontrol(guiel.cPanel(4),'Style','PushButton','Units','pixels',...
	'Position',[2*cnst.icwidth 0 cnst.icwidth cnst.icwidth],'Enable','on','CData',iccursor,'TooltipString',guiel.cursorTt,...
    'CallBack',['set(guiel.APPWINDOW,''PointerShapeCData'',cnst.dataCursor,''WindowButtonMotionFcn'',''eval(codeblk.ZoomPointerControl);''); '...
                'if ishandle(guiel.CursorWindow) delete(guiel.CursorWindow);'...
                'set(guiel.hAX(1),''ButtonDownFcn'','''');'...
                'set(guiel.oTrace(3),''XData'',0,''YData'',0,''Marker'',''none''); end;'...
                'set(guiel.hAX(1),''ButtonDownFcn'',''cursorBar''); set(guiel.zoomzoom,''Enable'',''on'');']); 
 
clear iczoomin iczoomout iccursor;


%===================================================================
%=====CLOSE EXPERIMENT==============================================
%===================================================================
guiel.hPB(7)=uicontrol(guiel.ControlPanel,'Style','PushButton','Units','pixels',...
	'Position',[cnst.hmargin+cnst.bhspace cnst.bvspace cnst.bwidth cnst.bheight],...
	'String','Close Experiment','fontsize',10,'FontWeight','Demi','CallBack','eval(codeblk.SHUT_DOWN);');


%===================================================================
%=====Help Files====================================================
%===================================================================
set(guiel.h_MN(04),'Callback','alertBox(guiel.APPWINDOW,[(cnst.displaywidth+cnst.cpwidth)/2-200 (cnst.displayheight)/2-250 400 500],guiel.helptext,''Help Me'');'); 


%===================================================================
%=====CODE BLOCK DEFINITIONS========================================
%===================================================================

codeblk.OPEN_MDL_FCN = ['if  isempty(find_system(''Name'',vars.simfilename))'...
                    'open_system(vars.simfilename);'...
                    'set_param(vars.simfilename,''CloseFcn'','...
                    '[''save_system(vars.simfilename); sim(vars.simfilename);'...
                    'vars.simtime = str2num(get_param(vars.simfilename,''''StopTime''''));'...
                    'set(guiel.hSimtime,''''String'''',num2str(vars.simtime));'...
                    'if strcmp(vars.simfilename(end-2:end),''''sim'''') '...
                        'vars.L= motorParams(1,1); vars.R= motorParams(1,2); vars.J= motorParams(1,3);'...
                        'vars.b= motorParams(1,4); vars.Kb= motorParams(1,5); vars.Km= motorParams(1,6);'...
                        'if (min(size(motorParams)) == 9) '...
                            'vars.P= motorParams(1,7); vars.I= motorParams(1,8); vars.D= motorParams(1,9);'...
                        'else;'...
                            'vars.P= NaN; vars.I= NaN; vars.D= NaN;'...
                        'end;'...
                        'clear motorParams; [vars.numericTF vars.symbolTF] = calcTF2(vars);'...
                    'end,'']);'...
                    'set_param(vars.simfilename,''StopFcn'',''save_system(vars.simfilename);'');'...
                    'set_param(vars.simfilename,''StartTime'',''0.0'',''StopTime'',num2str(vars.simtime));'...
                    'end'];
                
codeblk.CLOSE_MDL_FCN = ['if  ~isempty(find_system(''Name'',vars.simfilename))'...
                    'vars.simtime = str2num(get_param(vars.simfilename,''StopTime''));'...
                    'set(guiel.hSimtime,''String'',num2str(vars.simtime));'...
                    'save_system(vars.simfilename);'...
                    'close_system(vars.simfilename);'...
                    'end'];

codeblk.SHUT_DOWN = ['eval(codeblk.stopTimer);'...
                'del_globalvars; bdclose(''all''); rmpath(genpath(cnst.p));'...
                'delete(guiel.APPWINDOW);'...
                'clear input output velocity voltage magnet anim rd codeblk hght tout vars cnst guiel;'];

codeblk.stopTimer = 'if isvalid(vars.pTimer) stop(vars.pTimer); delete(vars.pTimer); end, vars.enableanimation = 0;';

codeblk.cleanButtonFcns = 'set(guiel.APPWINDOW,''WindowButtonMotionFcn'','''',''KeyPressFcn'','''',''WindowButtonUpFcn'','''',''Pointer'',''arrow'');';

codeblk.ZoomPointerControl=[...
    'vars.axlims = [get(guiel.hAX(1),''XLim'') get(guiel.hAX(1),''YLim'')];'...
    'vars.CurrentPoint=get(guiel.hAX(1),''currentpoint'');',...
     'if ((vars.CurrentPoint(1,1) < vars.axlims(1)) | (vars.CurrentPoint(1,1) > vars.axlims(2)) | (vars.CurrentPoint(1,2) < vars.axlims(3)) | (vars.CurrentPoint(1,2) > vars.axlims(4)));',...
     '          set(guiel.APPWINDOW,''pointer'',''arrow'');',...
     'else;',...
     '          set(guiel.APPWINDOW,''pointer'',''custom'');',...
     'end;',...
     ];

 codeblk.calcTF = [...
  'if (vars.L ~= 0) '...
    'vars.num3rdO = [1 vars.P/vars.D vars.I/vars.D];'...
    'vars.den3rdO = [1, vars.b/vars.J + vars.R/vars.L + vars.D*2*vars.Km/vars.J/vars.L,...'...
        '(vars.R*vars.b + vars.P*2*vars.Km + 2*vars.Kb)/vars.J/vars.L, vars.I*2*vars.Km/vars.J/vars.L];'...
    'vars.gain3rdO = [2*vars.Km*vars.D/vars.J/vars.L];'...
    'vars.tf3rdO = tf(vars.num3rdO, vars.den3rdO);'...
 'else,'...
    'vars.num3rdO = 0;'...
    'vars.den3rdO = 0;'...
    'vars.gain3rdO = 0;'...
    'vars.tf3rdO = 0;'...
'end;'...
    'vars.num2ndO = [1 vars.P/vars.D vars.I/vars.D];'...
    'vars.den2ndO = [1, (vars.R*vars.b + vars.P*2*vars.Km + vars.Kb*vars.Km)/(vars.J*vars.R + vars.D*2*vars.Km),...'...
                    'vars.I*vars.Km*2/(vars.J*vars.R + vars.D*2*vars.Km)];'...
    'vars.gain2ndO = [vars.D*2*vars.Km/(vars.J*vars.R + vars.D*2*vars.Km)];'...
    'vars.tf2ndO = tf(vars.num2ndO, vars.den2ndO);'];
 
%just set this as the figures pointershapecdata, I tried to play with this
%to make it look nicer, it would be nice if it looked more round.
cnst.MagnifyingGlass = [NaN NaN NaN NaN   1   1   1   1 NaN NaN NaN NaN NaN NaN NaN NaN;...
                        NaN NaN   1   1 NaN NaN NaN NaN   1   1 NaN NaN NaN NaN NaN NaN;...
                        NaN   1 NaN NaN NaN NaN NaN NaN NaN NaN   1 NaN NaN NaN NaN NaN;...
                        NaN   1 NaN NaN NaN NaN NaN NaN NaN NaN   1 NaN NaN NaN NaN NaN;...
                          1 NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN   1 NaN NaN NaN NaN;...
                          1 NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN   1 NaN NaN NaN NaN;...
                          1 NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN   1 NaN NaN NaN NaN;...
                          1 NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN   1 NaN NaN NaN NaN;...
                        NaN   1 NaN NaN NaN NaN NaN NaN NaN NaN   1 NaN NaN NaN NaN NaN;...
                        NaN   1 NaN NaN NaN NaN NaN NaN NaN NaN   1 NaN NaN NaN NaN NaN;...
                        NaN NaN   1   1 NaN NaN NaN NaN   1   1   1   1 NaN NaN NaN NaN;...
                        NaN NaN NaN NaN   1   1   1   1 NaN NaN   1   1   1 NaN NaN NaN;...
                        NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN   1   1   1 NaN NaN;...
                        NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN   1   1   1 NaN;...
                        NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN   1   1   1;...
                        NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN   1 NaN];
                    
cnst.dataCursor = [  NaN NaN  1  NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN;...
                     NaN NaN  1  NaN NaN NaN 1    1   1  NaN NaN NaN NaN NaN NaN NaN;...
                     1    1   1   1   1  NaN 1    2   1    1 NaN NaN NaN NaN NaN NaN;...
                     NaN NaN  1  NaN NaN NaN NaN NaN  1   2   1   1  NaN NaN NaN NaN;...
                     NaN NaN  1  NaN NaN NaN NaN NaN  1   2   2   1   1  NaN NaN NaN;...
                     NaN NaN NaN NaN NaN NaN NaN NaN  1   2   2   1   1   1  NaN NaN;...
                     NaN NaN NaN  1  NaN NaN NaN  1   1   2   1   1   2   1   1  NaN;...
                     NaN NaN  1   2   1  NaN NaN  1   1   2   2   1   2   1   2   1 ;...
                     NaN NaN  1   2   2    1  1   1   2   2   2   2   2   1   2   1 ;...
                     NaN NaN NaN  1   2   2   2   2   2   2   2   2   2   2   2   1 ;...
                     NaN NaN NaN  1   2   2   2   2   2   2   2   2   2   2   2   1 ;...
                     NaN NaN NaN NaN  1   2   2   2   2   2   2   2   2   2   2   1 ;...
                     NaN NaN NaN NaN NaN  1   2   2   2   2   2   2   2   2   1  NaN;...  
                     NaN NaN NaN NaN NaN NaN  1   1   2   2   2   2   2   1  NaN NaN;...
                     NaN NaN NaN NaN NaN NaN NaN  1   2   2   2   2   2   1  NaN NaN;...
                     NaN NaN NaN NaN NaN NaN NaN NaN   1  1   1   1   1  NaN NaN NaN];
